from nosessionapp import handlers
from firenado import tornadoweb


class NosessionappComponent(tornadoweb.TornadoComponent):

    def get_handlers(self):
        return [
            (r'/', handlers.IndexHandler),
        ]



#!/usr/bin/env python
#
# Copyright 2015-2017 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import tornado.web


class Header1(tornado.web.UIModule):

    def render(self, header):
        return self.render_string('header1.html', header=header)


class LoginErrorMessage(tornado.web.UIModule):

    def render(self, key):

        if self.handler.session.has('login_errors'):
            errors = self.handler.session.get('login_errors')
            if key in errors:
                template = "testapp:uimodules/login_error_message.html"
                return self.render_string(template, message=errors[key])
        return ""



import multiapp.handlers
import firenado.tornadoweb


class App1Component(firenado.tornadoweb.TornadoComponent):

    def get_handlers(self):
        if self.is_current_app():
            return [
                (r'/', multiapp.handlers.App1IndexHandler),
            ]
        return []


class App2Component(firenado.tornadoweb.TornadoComponent):

    def get_handlers(self):
        if self.is_current_app():
            return [
                (r'/', multiapp.handlers.App2IndexHandler),
            ]
        return []



# Copyright 2015-2024 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
from firenado.service import FirenadoService
from firenado.launcher import ProcessLauncher, TornadoLauncher
from tornado.testing import (bind_unused_port, AsyncTestCase,
                             AsyncHTTPTestCase)
from unittest import TestCase


def get_event_loop():
    """ Return a new event asyncio evenvent loop with trigerring a
    depreciation warning from Python 10.

    :return: A event loop
    """
    loop = asyncio.get_event_loop_policy().get_event_loop()
    return loop if loop else asyncio.new_event_loop()


class ServiceTestCase(TestCase):

    def setUp(self):
        """ Call the configure data connection method
        """
        self.configure_data_connected()

    def configure_data_connected(self):
        """Should be overridden with the configutation of the data connected
        instance
        """
        raise NotImplementedError()

    @property
    def data_connected(self):
        """Should be overridden by subclasses to return a data connected
        instance
        """
        raise NotImplementedError()


class TestableService(FirenadoService):
    """ Serves a data connected method directly.
    When decorating a data connected directly the service must return the
    consumer.
    """
    pass


class TornadoAsyncTestCase(AsyncTestCase):

    @property
    def launcher(self) -> ProcessLauncher:
        return self.__launcher

    def get_launcher(self) -> ProcessLauncher:
        """Should be overridden by subclasses to return a
        `firenado.launcher.TornadoLauncher`.
        """
        raise NotImplementedError()

    def http_port(self) -> int:
        return self.__port

    def setUp(self) -> None:
        import logging
        super().setUp()
        sock, port = bind_unused_port()
        sock.close()
        self.__port = port
        self.__launcher = self.get_launcher()
        self.__launcher.configure_logging(level=logging.ERROR)
        self.__launcher.port = self.__port
        self.__launcher.load()
        asyncio.run(self.__launcher.launch())

    def tearDown(self) -> None:
        self.__launcher.shutdown()
        super().tearDown()


class TornadoAsyncHTTPTestCase(AsyncHTTPTestCase):

    def get_launcher(self) -> TornadoLauncher:
        """Should be overridden by subclasses to return a
        `firenado.launcher.TornadoLauncher`.
        """
        raise NotImplementedError()

    def get_log_level(self):
        return None

    def get_app(self):
        import logging
        launcher = self.get_launcher()
        launcher.load()
        if self.get_log_level() is not None:
            launcher.configure_logging(level=self.get_log_level())
        else:
            launcher.configure_logging(level=logging.WARN)
        return launcher.application



#!/usr/bin/env python
#
# Copyright 2015-2016 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import firenado.tornadoweb
import logging
import tornado.escape
import uuid


class MainHandler(firenado.tornadoweb.TornadoHandler):

    def get(self):
        self.render("index.html", messages=ChatSocketHandler.cache)


class ChatSocketHandler(firenado.tornadoweb.TornadoWebSocketHandler):

    waiters = set()
    cache = []
    cache_size = 200

    def get_compression_options(self):
        # Non-None enables compression with default options.
        return {}

    def open(self):
        ChatSocketHandler.waiters.add(self)

    def on_close(self):
        ChatSocketHandler.waiters.remove(self)

    @classmethod
    def update_cache(cls, chat):
        cls.cache.append(chat)
        if len(cls.cache) > cls.cache_size:
            cls.cache = cls.cache[-cls.cache_size:]

    @classmethod
    def send_updates(cls, chat):
        logging.info("sending message to %d waiters", len(cls.waiters))
        for waiter in cls.waiters:
            try:
                waiter.write_message(chat)
            except:
                logging.error("Error sending message", exc_info=True)

    def on_message(self, message):
        logging.info("got message %r", message)
        parsed = tornado.escape.json_decode(message)
        chat = {
            "id": str(uuid.uuid4()),
            "body": parsed["body"],
        }
        chat["html"] = tornado.escape.to_basestring(
            self.render_string("message.html", message=chat))

        ChatSocketHandler.update_cache(chat)
        ChatSocketHandler.send_updates(chat)



#!/usr/bin/env python
#
# Copyright 2015-2016 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import firenado.security
import firenado.tornadoweb
import tornado
import tornado.auth
import tornado.web


class FacebookHandlerMixin:

    def get_current_user(self):
        user_json = self.session.get("fbdemo_user")
        if not user_json:
            return None
        return tornado.escape.json_decode(user_json)


class MainHandler(FacebookHandlerMixin, firenado.tornadoweb.TornadoHandler,
                  tornado.auth.FacebookGraphMixin):

    @firenado.security.authenticated
    @tornado.web.asynchronous
    def get(self):
        self.facebook_request("/me/feed", self._on_stream,
                              access_token=self.current_user["access_token"])

    def _on_stream(self, stream):
        if stream is None:
            # Session may have expired
            self.redirect("/auth/login")
            return
        self.render("stream.html", stream=stream)


class AuthLoginHandler(FacebookHandlerMixin,
                       firenado.tornadoweb.TornadoHandler,
                       tornado.auth.FacebookGraphMixin):

    @tornado.web.asynchronous
    def get(self):
        my_url = (self.request.protocol + "://" + self.request.host +
                  "/auth/login?next=" +
                  tornado.escape.url_escape(self.get_argument("next", "/")))
        if self.get_argument("code", False):
            self.get_authenticated_user(
                redirect_uri=my_url,
                client_id=self.settings["facebook_api_key"],
                client_secret=self.settings["facebook_secret"],
                code=self.get_argument("code"),
                callback=self._on_auth)
            return
        self.authorize_redirect(redirect_uri=my_url,
                                client_id=self.settings["facebook_api_key"],
                                extra_params={"scope": "user_posts"})

    def _on_auth(self, user):
        if not user:
            raise tornado.web.HTTPError(500, "Facebook auth failed")
        self.session.set("fbdemo_user", tornado.escape.json_encode(user))
        self.redirect(self.get_argument("next", "/"))


class AuthLogoutHandler(firenado.tornadoweb.TornadoHandler,
                        tornado.auth.FacebookGraphMixin):

    def get(self):
        self.session.delete("fbdemo_user")
        self.redirect(self.get_argument("next", "/"))



#!/usr/bin/env python
#
# Copyright 2015-2021 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from . import tasks
from firenado.management import ManagementCommand
from tornado import template
import os

TESTAPP_ROOT = os.path.realpath(os.path.join(os.path.dirname(__file__), ".."))
loader = template.Loader(os.path.join(TESTAPP_ROOT, "templates", "management"))


ManagementCommand(
    "testapp", "Testapp related commands",
    loader.load("testapp_command_help.txt"), category="Testapp",
    sub_commands=[
        ManagementCommand("command1", "Execute the command 1", "",
                          tasks=tasks.TestAppCommand1Task),
        ManagementCommand("command2", "Execute the command 2", "",
                          tasks=tasks.TestAppCommand2Task)
    ])



import feed.handlers
import firenado.tornadoweb

class FeedComponent(firenado.tornadoweb.TornadoComponent):

    def get_handlers(self):
        return [
            (r'/', feed.handlers.IndexHandler),
        ]



# Copyright 2015-2023 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
from inspect import isfunction, ismethod
import logging
from sqlalchemy import inspect, func, text

logger = logging.getLogger(__name__)


def base_to_dict(base, columns=None):
    """ Returns a dict inherited from Base. If keys is provided it will only
    add to the dict the columns provided.
    :param base: An orm object inherited from Base
    :param columns: Columns to be added to the dict
    :return dict: A dictionary representing the Base object.
    """
    if columns is None:
        return {c.key: getattr(base, c.key)
                for c in inspect(base).mapper.column_attrs}
    return {c.key: getattr(base, c.key)
            for c in inspect(base).mapper.column_attrs if c.key in columns}


def fast_count(query):
    """
    based on https://gist.github.com/hest/8798884
    :return:
    """
    count_statement = query.statement.with_only_columns(
        [func.count()]).order_by(None)
    count = query.session.execute(count_statement).scalar()
    return count


def run_script(script_path, session, handle_command=None, handle_line=None):
    """ Run a script file using a valid sqlalchemy session.

    Based on https://bit.ly/2CToAhY.
    See also sqlalchemy transaction control: https://bit.ly/2yKso0A

    :param script_path: The path where the script is located
    :param session: A sqlalchemy session to execute the sql commands from the
    script
    :param handle_command: Function to handle a valid command
    :param handle_line: Function to handle a valid line
    :return:
    """
    logger.debug("Opening script %s.", script_path)
    with open(script_path, "r") as stream:
        sql_command = ""
        for line in stream:
            # Ignore commented lines
            if not line.startswith("--") and line.strip("\n"):
                # Append line to the command string
                if handle_line is not None:
                    logger.debug("Calling the handle line function for: %s.",
                                 line)
                    line = handle_line(line)
                sql_command = "%s%s" % (sql_command, line.strip("\n"))
                # If the command string ends with ";", it is a full statement
                if sql_command.endswith(";"):
                    # Try to execute statement and commit it
                    try:
                        if handle_command is not None:
                            logger.debug("Calling the handle command function "
                                         "for: %s.", sql_command)
                            sql_command = handle_command(sql_command)
                        session.execute(text(sql_command))
                    # Assert in case of error
                    except Exception as e:
                        session.rollback()
                        raise e
                    # Finally, clear command string
                    finally:
                        sql_command = ""
    session.commit()


def with_session(*args, **kwargs):
    """ This decorator will add an existing sqlalchemy session to the method
    being decorated or create a new sqlalchemy session to be used by the
    method."""
    service = None
    if len(args) > 0:
        service = args[0]

    def method_wrapper(method):
        @functools.wraps(method)
        def wrapper(self, *method_args, **method_kwargs):
            session = method_kwargs.get("session")
            close = kwargs.get("close", False)
            close = method_kwargs.get("close", close)
            if not session:
                # If a new session is created we must close it after running
                # the method
                close = True
                data_source = kwargs.get("data_source")
                data_source = method_kwargs.get("data_source", data_source)
                if not data_source:
                    hasattr(self, "default_data_source")
                    if hasattr(self, "default_data_source"):
                        if ismethod(self.default_data_source):
                            data_source = self.default_data_source()
                        else:
                            data_source = self.default_data_source
                try:
                    if isinstance(data_source, str):
                        ds = self.get_data_source(data_source)
                        method_kwargs['data_source'] = data_source
                        session = ds.session
                    else:
                        session = data_source.session
                    method_kwargs['session'] = session
                except KeyError:
                    logger.exception("There is no datasource defined with "
                                     "index \"%s\" related to the service.",
                                     data_source)
            result = method(self, *method_args, **method_kwargs)
            if close:
                if not session:
                    logger.warning("No session was resolved.")
                logger.debug("Closing session %s.", session)
                if hasattr(session, "autoflush") and not session.autoflush:
                    session.flush()
                session.close()
            return result
        return wrapper
    # If the decorator has no parameter, I mean no parentesis, we need to wrap
    # the service variable again , instead of the service instance, we need to
    # deal with the method being decorated but as a <function> instace.
    if isfunction(service):
        @functools.wraps(None)
        def func_wrapper(_function):
            return method_wrapper(_function)
        return func_wrapper(service)
    return method_wrapper



import firenado.tornadoweb
from feedgen.feed import FeedGenerator
from datetime import datetime
import pytz



class IndexHandler(firenado.tornadoweb.TornadoHandler):

    def get(self):
        fg = FeedGenerator()
        fg.id("http://test.ts")
        fg.title("My Test Feed")
        fg.icon("https://avatars1.githubusercontent.com/u/715660?v=3&s=32")
        fg.author({'name': "The Author", 'email': "test@test.ts"})

        fg.link(href="http://example.org/index.atom?page=2", rel="next")

        fg.link(href="http://test.ts", rel="alternate")
        fg.logo("https://avatars1.githubusercontent.com/u/715660?v=3&s=32")
        fg.description("Este é o monstro do lago 1")
        fg.subtitle("This is an example feed!")
        fg.language("en-us")
        # Handle this:
        #< sy:updatePeriod > hourly < / sy:updatePeriod >
        #< sy:updateFrequency > 1 < / sy:updateFrequency >

        fg.lastBuildDate(datetime.now(pytz.timezone("America/Sao_Paulo")))

        fi = fg.add_item()
        fi.id("http://test.ts/id/1", )
        #fi.link(link="http://test.ts/id/1")
        fi.title("Monstro do Lago 1")
        fi.description("Este é o monstro do lago 1")
        fi.comments("http://test.ts/id/1/comments")
        fi.pubdate(datetime.now(pytz.timezone("America/Sao_Paulo")))

        fi = fg.add_item()
        fi.id("http://test.ts/id/2")
        fi.title("Monstro do Lago 2")
        fi.description("Este é o monstro do lago 2")
        fi.pubdate(datetime.now(pytz.timezone("America/Sao_Paulo")))

        #test = fg.atom_str(pretty=True)

        rss_str = fg.rss_str(pretty=True)
        self.set_header("Content-Type", 'application/xml; charset="utf-8"')
        #self.set_header("Content-Disposition",
        # "attachment; filename='test.xml'")
        self.write(rss_str)


        #if regexp.search(word) is not None:
        #    print
        #    'matched'
        if self.is_mobile():
            print("buu")
        else:
            print(self.request.headers["User-Agent"])

        #for zone in pytz.common_timezones:
        #    print(pytz.timezone(zone).localize(
        #        datetime(2002, 10, 27, 6, 0, 0)).strftime("%Z %z"), zone)



# Copyright 2015-2024 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tests import chdir_fixture_app, PROJECT_ROOT
from firenado.testing import TornadoAsyncHTTPTestCase, TornadoAsyncTestCase
from firenado.launcher import ProcessLauncher, TornadoLauncher
from tornado.httpclient import AsyncHTTPClient
from tornado.testing import gen_test


class AsyncTestCase(TornadoAsyncTestCase):

    def get_launcher(self):
        application_dir = chdir_fixture_app("launcherapp")
        return ProcessLauncher(
            dir=application_dir, path=PROJECT_ROOT)

    @gen_test
    async def test_get(self):
        http_client = AsyncHTTPClient()
        try:
            response = await http_client.fetch(
                    f"http://localhost:{self.http_port()}/")
        except Exception as e:
            raise e
        self.assertEqual(response.body, b"Get output")

    @gen_test
    async def test_post(self):
        http_client = AsyncHTTPClient()
        try:
            response = await http_client.fetch(
                f"http://localhost:{self.http_port()}/",
                body="",
                method="POST"
            )
        except Exception as e:
            raise e
        self.assertEqual(response.body, b"Post output")


class AsyncHTTPTestCase(TornadoAsyncHTTPTestCase):

    def get_launcher(self):
        application_dir = chdir_fixture_app("launcherapp")
        return TornadoLauncher(
            dir=application_dir, path=PROJECT_ROOT)

    def test_get(self):
        response = self.fetch("/")
        # print(response.code)
        self.assertEqual(response.body, b"Get output")

    def test_post(self):
        response = self.fetch("/", body="", method="POST")
        # print(response.code)
        self.assertEqual(response.body, b"Post output")



#!/usr/bin/env python
#
# Copyright 2015-2016 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import firenado.tornadoweb
import tornado.escape
from tornado import gen
import uuid


class MainHandler(firenado.tornadoweb.TornadoHandler):

    def get(self):
        self.render("index.html",
                    messages=self.component.message_buffer.cache)


class MessageNewHandler(firenado.tornadoweb.TornadoHandler):

    def post(self):
        message = {
            "id": str(uuid.uuid4()),
            "body": self.get_argument("body"),
        }
        # to_basestring is necessary for Python 3's json encoder,
        # which doesn't accept byte strings.
        message["html"] = tornado.escape.to_basestring(
            self.render_string("message.html", message=message))
        if self.get_argument("next", None):
            self.redirect(self.get_argument("next"))
        else:
            self.write(message)
        self.component.message_buffer.new_messages([message])


class MessageUpdatesHandler(firenado.tornadoweb.TornadoHandler):

    @gen.coroutine
    def post(self):
        cursor = self.get_argument("cursor", None)
        # Save the future returned by wait_for_messages so we can cancel
        # it in wait_for_messages
        self.future = self.component.message_buffer.wait_for_messages(
            cursor=cursor)
        messages = yield self.future
        if self.request.connection.stream.closed():
            return
        self.write(dict(messages=messages))

    def on_connection_close(self):
        self.component.message_buffer.cancel_wait(self.future)



#!/usr/bin/env python
#
# Copyright 2015-2016 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import chatdemo.handlers
import firenado.tornadoweb
import logging
from tornado.concurrent import Future


class MessageBuffer(object):

    def __init__(self):
        self.waiters = set()
        self.cache = []
        self.cache_size = 200

    def wait_for_messages(self, cursor=None):
        # Construct a Future to return to our caller.  This allows
        # wait_for_messages to be yielded from a coroutine even though
        # it is not a coroutine itself.  We will set the result of the
        # Future when results are available.
        result_future = Future()
        if cursor:
            new_count = 0
            for msg in reversed(self.cache):
                if msg['id'] == cursor:
                    break
                new_count += 1
            if new_count:
                result_future.set_result(self.cache[-new_count:])
                return result_future
        self.waiters.add(result_future)
        return result_future

    def cancel_wait(self, future):
        self.waiters.remove(future)
        # Set an empty result to unblock any coroutines waiting.
        future.set_result([])

    def new_messages(self, messages):
        logging.info("Sending new message to %r listeners", len(self.waiters))
        for future in self.waiters:
            future.set_result(messages)
        self.waiters = set()
        self.cache.extend(messages)
        if len(self.cache) > self.cache_size:
            self.cache = self.cache[-self.cache_size:]


class ChatdemoComponent(firenado.tornadoweb.TornadoComponent):

    def __init__(self, name, application):
        self.message_buffer = MessageBuffer()
        super(ChatdemoComponent, self).__init__(name, application)

    def get_handlers(self):
        return [
            (r'/', chatdemo.handlers.MainHandler),
            (r"/a/message/new", chatdemo.handlers.MessageNewHandler),
            (r"/a/message/updates", chatdemo.handlers. MessageUpdatesHandler),
        ]



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2022 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from firenado.schedule import ScheduledJob
from tornado.gen import sleep


class PrintTestJob(ScheduledJob):

    def __init__(self, scheduler, **kwargs):
        super(PrintTestJob, self).__init__(scheduler, **kwargs)
        self._custom_property = kwargs.get('custom_property', None)

    def run(self):
        print("This is the custom property value: %s" % self._custom_property)


class PrintAsyncTestJob(ScheduledJob):

    def __init__(self, scheduler, **kwargs):
        super(PrintAsyncTestJob, self).__init__(scheduler, **kwargs)
        self._custom_property = kwargs.get('custom_property', None)

    async def run(self):
        print("This is the custom property value from async job: %s" %
              self._custom_property)
        await sleep(1)


class ErroredJob(ScheduledJob):

    def __init__(self, scheduler, **kwargs):
        super(ErroredJob, self).__init__(scheduler, **kwargs)
        self._custom_property = kwargs.get('custom_property', None)

    def run(self):
        error_result = 1/0
        print("This is the custom property value: %s" % self._custom_property)


class ErroredAsyncJob(ScheduledJob):

    def __init__(self, scheduler, **kwargs):
        super(ErroredAsyncJob, self).__init__(scheduler, **kwargs)
        self._custom_property = kwargs.get('custom_property', None)

    async def run(self):
        error_result = 1/0
        print("This is the custom property value: %s" % self._custom_property)
        await sleep(1)



#!/usr/bin/env python
#
# Copyright 2015-2016 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import firenado.security
import firenado.tornadoweb

from tornado.auth import GoogleOAuth2Mixin
from tornado.escape import json_decode, json_encode

from tornado import gen


class GoogleHandlerMixin:

    COOKIE_NAME = 'googledemo_user'

    def get_current_user(self):
        user_json = self.session.get(self.COOKIE_NAME)
        print(user_json)
        if not user_json:
            return None
        return json_decode(user_json)


class MainHandler(GoogleHandlerMixin, firenado.tornadoweb.TornadoHandler,
                  GoogleOAuth2Mixin):

    @firenado.security.authenticated
    @gen.coroutine
    def get(self):
        self.write(self.get_current_user())


class LoginHandler(GoogleHandlerMixin, firenado.tornadoweb.TornadoHandler,
                   GoogleOAuth2Mixin):
    @gen.coroutine
    def get(self):
        my_redirect_url = "%s://%s/login" % (self.request.protocol,
                                              self.request.host)
        if self.get_argument('code', False):
            access = yield self.get_authenticated_user(
                redirect_uri=my_redirect_url,
                code=self.get_argument('code'))
            user = yield self.oauth2_request(
                "https://www.googleapis.com/oauth2/v1/userinfo",
                access_token=access["access_token"])
            # Save the user and access token with
            # e.g. set_secure_cookie.
            self.session.set(self.COOKIE_NAME, json_encode(user))
            self.redirect(self.get_argument('next', '/'))
        else:
            yield self.authorize_redirect(
                redirect_uri=my_redirect_url,
                client_id=self.settings['google_oauth']['key'],
                client_secret=self.settings['google_oauth']['secret'],
                scope=['profile', 'email'],
                response_type='code',
                extra_params={'approval_prompt': 'auto'})


class LogoutHandler(GoogleHandlerMixin, firenado.tornadoweb.TornadoHandler):

    def get(self):
        self.session.delete(self.COOKIE_NAME)



#!/usr/bin/env python
#
# Copyright 2015-2016 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import firenado.security
import firenado.tornadoweb

from tornado.auth import TwitterMixin
from tornado.escape import json_decode, json_encode

from tornado import gen


class TwitterHandlerMixin:

    COOKIE_NAME = 'twitterdemo_user'

    def get_current_user(self):
        user_json = self.session.get(self.COOKIE_NAME)
        print(user_json)
        if not user_json:
            return None
        return json_decode(user_json)


class MainHandler(TwitterHandlerMixin, firenado.tornadoweb.TornadoHandler, TwitterMixin):

    @firenado.security.authenticated
    @gen.coroutine
    def get(self):
        timeline = yield self.twitter_request(
            '/statuses/home_timeline',
            access_token=self.current_user['access_token'])
        self.render('home.html', timeline=timeline)


class LoginHandler(TwitterHandlerMixin, firenado.tornadoweb.TornadoHandler, TwitterMixin):
    @gen.coroutine
    def get(self):
        if self.get_argument('oauth_token', None):
            user = yield self.get_authenticated_user()
            del user["description"]
            self.session.set(self.COOKIE_NAME, json_encode(user))
            self.redirect(self.get_argument('next', '/'))
        else:
            yield self.authorize_redirect(callback_uri=self.request.full_url())


class LogoutHandler(TwitterHandlerMixin, firenado.tornadoweb.TornadoHandler):

    def get(self):
        self.session.delete(self.COOKIE_NAME)



from . import handlers
from firenado import tornadoweb
from firenado.launcher import ProcessLauncher
import os


class LauncherComponent(tornadoweb.TornadoComponent):

    def __init__(self, name, application):
        super(LauncherComponent, self).__init__(name, application)
        self.launcher_path = os.path.abspath(os.path.dirname(__file__))
        self.charge_path = os.path.join(self.launcher_path, "charge")
        self.launcher = None
        self.launcher_callback = None

    def get_handlers(self):
        return [
            (r'/', handlers.IndexHandler),
        ]

    def initialize(self):
        import tornado.ioloop
        self.launcher_callback = tornado.ioloop.IOLoop.current().add_callback(
            callback=self.launch_charge
        )

    async def launch_charge(self):
        import sys
        os.chdir(self.charge_path)
        self.launcher = ProcessLauncher(dir=self.charge_path,
                                        logfile=sys.stderr)
        self.launcher.load()
        await self.launcher.launch()
        os.chdir(self.launcher_path)

    def shutdown(self):
        self.launcher.shutdown()



import firenado.tornadoweb


class App1IndexHandler(firenado.tornadoweb.TornadoHandler):

    def get(self):
        self.write("App1 IndexHandler output")


class App2IndexHandler(firenado.tornadoweb.TornadoHandler):

    def get(self):
        self.write("App2 IndexHandler output")



#!/usr/bin/env python
#
# Copyright 2015-2021 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from . import handlers, services, uimodules
from firenado import tornadoweb
from firenado import data, service
import logging
from typing import Any

logger = logging.getLogger(__name__)


class ComponentCustomErrorHandler(tornadoweb.TornadoErrorHandler):

    def handle_error(self, request: tornadoweb.TornadoHandler,
                     status_code: int, **kwargs: Any) -> None:
        request.set_status(status_code)
        request.render("error.html", http_error=kwargs.get('exc_info')[1])


class TestappComponent(tornadoweb.TornadoComponent):

    user_service: services.LoginService

    def __init__(self, name, application):
        super(TestappComponent, self).__init__(name, application)
        self.user_service = None

    def get_error_handler(self) -> tornadoweb.TornadoErrorHandler:
        return ComponentCustomErrorHandler(self)

    def get_handlers(self):
        import firenado.conf
        default_login = firenado.conf.app['login']['urls']['default']
        return [
            (r"/", handlers.IndexHandler),
            (r"/async/timeout", handlers.AsyncTimeoutHandler),
            (r"/component/error", handlers.ComponentErrorHandler),
            (r"/handler/error", handlers.HandlerErrorHandler),
            (r"/session/counter", handlers.SessionCounterHandler),
            (r"/session/config", handlers.SessionConfigHandler),
            (r"/pagination", handlers.PaginationHandler),
            (r"/%s" % default_login, handlers.LoginHandler),
            (r"/%s" % default_login, handlers.LoginHandler),
            (r"/logout", handlers.LogoutHandler),
            (r"/private", handlers.PrivateHandler),
        ]

    def get_ui_modules(self):
        return uimodules

    def initialize(self):
        import firenado.conf
        firenado.conf.app['login']['urls']['buga'] = 'buga'
        data_source_conf = {
            'connector': "sqlalchemy",
            'url': "mysql+pymysql://root@localhost:3306/test",
            # 'isolation_level': 'REPEATABLE READ',
            'pool': {
                'size': 10,
                'max_overflow': 10,
                # 'pool_recycle': 400
            }
        }
        data_source_name = "test"
        data_source = data.config_to_data_source(data_source_name,
                                                 data_source_conf,
                                                 self.application)
        self.application.set_data_source(data_source_name, data_source)

    @service.served_by(services.UserService)
    def install(self):
        from sqlalchemy.engine import Engine
        """  Installing test database
        """
        from testapp.models import Base
        print('Installing Testapp App...')
        print('Creating App ...')
        engine: Engine = self.application.get_data_source(
            'test').engine
        engine.echo = True
        with engine.connect() as conn:
            # Dropping all
            # TODO Not to drop all if something is installed right?
            Base.metadata.drop_all(conn)
            # Creating database
            Base.metadata.create_all(conn)
            conn.commit()
        self.user_service.create({
            'username': "Test",
            'first_name': "Test",
            'last_name': "User",
            'password': "testpass",
            'email': "test@test.ts"
        })

    @property
    def data_connected(self):
        return self.application

    def after_request(self, handler):
        logging.info("Doing something after handler's request: %s" % handler)

    def before_request(self, handler):
        logging.info("Doing something before handler's request: %s" % handler)



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2024 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .services import LoginService, UserService

import firenado.conf
from firenado import security, service, tornadoweb
from firenado.util.sqlalchemy_util import base_to_dict
import hashlib
from tornado import escape, gen
from tornado.web import HTTPError
from typing import Any
import logging

logger = logging.getLogger(__name__)


class AuthHandler:

    def get_current_user(self):
        from tornado.escape import json_decode
        user_data = self.session.get("user")
        if user_data:
            return json_decode(user_data)
        return None


class HandlerCustomErrorHandler(tornadoweb.TornadoErrorHandler):

    def handle_error(self, request: tornadoweb.TornadoHandler,
                     status_code: int, **kwargs: Any) -> None:
        request.set_status(status_code)
        request.render("error.html", http_error=kwargs.get('exc_info')[1])


class AsyncTimeoutHandler(tornadoweb.TornadoHandler):

    def timed_out(self):
        self.render("async_timeout.html")

    @gen.coroutine
    def get(self):
        from tornado.util import TimeoutError
        try:
            yield gen.with_timeout(2, gen.sleep(5))
            self.write("This will never be reached!!")
        except TimeoutError as te:
            logger.warning(te.__repr__())
            self.timed_out()


class IndexHandler(AuthHandler, tornadoweb.TornadoHandler):

    def get(self):
        import firenado.conf
        default_login = firenado.conf.app['login']['urls']['default']
        self.render("index.html", message="Hello world!!!",
                    login_url=default_login)


class ComponentErrorHandler(tornadoweb.TornadoHandler):

    def get(self):
        raise HTTPError(400, "This is an error thrown by my handler and "
                             "handled by the component's error handler.",
                        reason="ComponentCustomErrorHandler is set at the "
                               "handler's component and be used by default.")


class HandlerErrorHandler(tornadoweb.TornadoHandler):

    def get_error_handler(self) -> tornadoweb.TornadoErrorHandler:
        return HandlerCustomErrorHandler(self)

    def get(self):
        raise HTTPError(400, "This is an error thrown by my handler and "
                             "handled by the handler's error handler.",
                        reason="HandlerCustomErrorHandler is set at the "
                               "current handler overwriting the component's "
                               "custom error handler.")


class SessionConfigHandler(tornadoweb.TornadoHandler):

    def get(self):
        self.render("session_config.html", session_conf=firenado.conf.session)


class SessionCounterHandler(tornadoweb.TornadoHandler):

    def get(self):
        reset = self.get_argument("reset", False, True)
        if reset:
            self.session.delete('counter')
            self.redirect(self.get_rooted_path("/session/counter"))
            return None
        counter = 0
        if self.session.has('counter'):
            counter = self.session.get('counter')
        counter += 1
        self.session.set('counter', counter)
        self.render("session.html", session_value=counter)


class LoginHandler(AuthHandler, tornadoweb.TornadoHandler):

    login_service: LoginService
    user_service: UserService

    def get(self):
        default_login = firenado.conf.app['login']['urls']['default']
        errors = {}
        if self.authenticated():
            self.redirect(self.get_rooted_path("private"))
            return
        if self.session.has('login_errors'):
            errors = self.session.get('login_errors')
        self.render("login.html", errors=errors,
                    login_url=default_login)

    # you can user either the class rererence or string
    @service.with_service(LoginService)
    @service.with_service("testapp.services.UserService")
    def post(self):
        self.session.delete('login_errors')
        default_login = firenado.conf.app['login']['urls']['default']
        username = self.get_argument('username')
        password = self.get_argument('password')
        errors = {}
        if username == "":
            errors['username'] = "Please inform the username"
        if password == "":
            errors['password'] = "Please inform the password"

        if not errors:
            if not self.login_service.is_valid(username, password):
                errors['fail'] = "Invalid login"
            else:
                user = self.user_service.by_username(username)
                user_data = base_to_dict(user, ['id', 'username', 'password',
                                                'first_name', 'last_name',
                                                'email'])
                # Shhhhh!!!! this is a secret.
                user_data['password'] = hashlib.sha256(
                    "_".join(user_data['password']).encode('ascii')
                ).hexdigest()
                self.session.set("user", escape.json_encode(user_data))
                self.redirect(self.get_rooted_path("private"))

        if errors:
            self.session.set('login_errors', errors)
            self.redirect(self.get_rooted_path(default_login))

    def after_request(self):
        logging.info("Doing something after the login handler's request.")

    def before_request(self):
        logging.info("Doing something before the login handler's request.")


class LogoutHandler(AuthHandler, tornadoweb.TornadoHandler):

    def get(self):
        default_login = firenado.conf.app['login']['urls']['default']
        if self.authenticated():
            self.session.destroy(self)
            self.redirect(self.get_rooted_path("private"))
        else:
            self.redirect(self.get_rooted_path(default_login))
        return


class PrivateHandler(AuthHandler, tornadoweb.TornadoHandler):

    @security.authenticated
    def get(self):
        user_data = escape.json_decode(self.session.get("user"))
        self.render("private.html", user_data=user_data)


class PaginationHandler(tornadoweb.TornadoHandler):

    def get(self):
        pag_argument = "pag"
        page = self.get_argument(pag_argument, default=1)
        row_count = 316
        self.render("pagination.html", page=page, row_count=row_count,
                    pag_argument=pag_argument)



# Copyright 2015-2023 Flávio Gonçalves Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
from sqlalchemy import DateTime, Integer, String
from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column
from sqlalchemy.sql import text


class Base(DeclarativeBase):
    pass


class UserBase(Base):
    __tablename__ = "users"
    __table_args__ = {
        'mysql_engine': "InnoDB",
        'mysql_charset': "utf8",
    }

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    username: Mapped[str] = mapped_column(String(150), nullable=False)
    first_name: Mapped[str] = mapped_column(String(150), nullable=False)
    last_name: Mapped[str] = mapped_column(String(150), nullable=False)
    password: Mapped[str] = mapped_column(String(150), nullable=False)
    email: Mapped[str] = mapped_column(String(150), nullable=False)
    created: Mapped[datetime] = mapped_column(DateTime, nullable=False,
                                              server_default=text("now()"))
    modified: Mapped[datetime] = mapped_column(DateTime, nullable=False,
                                               server_default=text("now()"))



# Copyright 2015-2024 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License atex
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .models import UserBase
from firenado.service import FirenadoService, with_service
from firenado.sqlalchemy import with_session
from sqlalchemy import select
from sqlalchemy.orm import Session


def password_digest(pass_phrase):
    import hashlib
    m = hashlib.md5()
    m.update(pass_phrase.encode('utf-8'))
    return m.hexdigest()


class UserService(FirenadoService):

    @with_session(data_source="test")
    def create(self, user_data, **kwargs):
        session: Session = kwargs.get("session")
        user = UserBase()
        user.username = user_data['username']
        user.first_name = user_data['first_name']
        user.last_name = user_data['last_name']
        user.password = password_digest(user_data['password'])
        user.email = user_data['email']
        with session.begin():
            session.add(user)
        return user

    @with_session(data_source="test")
    def by_username(self, username, **kwargs) -> UserBase:
        session: Session = kwargs.get("session")
        session.begin()
        stmt = select(UserBase).where(UserBase.username == username)
        user = session.scalars(stmt).one_or_none()
        session.flush()
        return user


class LoginService(FirenadoService):

    user_service: UserService

    @with_service(UserService)
    def is_valid(self, username, password):
        """ Checks if challenge username and password matches
        username and password defined on the service constructor..

        Args:
            username: A challenge username
            password: A challenge password

        Returns: Returns true if challenge username and password matches
        username and password defined on the service constructor.

        """
        user = self.user_service.by_username(username)
        print(user)
        if user:
            if user.password == password_digest(password):
                return True
        return False



#!/usr/bin/env python
#
# Copyright 2015-2016 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4:

import tornado.web


class Header2(tornado.web.UIModule):

    def render(self, header):
        """ On this ui module example it is tested a second ui module package
        insertion to the application ui modules stack and, rendering a template
        from another component while rendering the ui module.

        :param header: The header string to be rendered by the module.
        :return: A header string
        """
        return self.render_string('testapp:header2.html', header=header)



from . import handlers
from firenado import schedule


class SchedappComponent(schedule.ScheduledTornadoComponent):

    def get_handlers(self):
        return [
            (r'/', handlers.IndexHandler),
        ]



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2015-2016 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from firenado.management import ManagementTask


logger = logging.getLogger(__name__)


class TestAppCommand1Task(ManagementTask):
    """ Run the command 1.
    """
    def run(self, namespace):
        logger.debug("Running TestApp Command 1")
        print("Test App Command 1")


class TestAppCommand2Task(ManagementTask):
    """ Run the command 2.
    """
    def run(self, namespace):
        logger.debug("Running TestApp Command 2")
        print("Test App Command 2")



import firenado.conf
from firenado import tornadoweb


class IndexHandler(tornadoweb.TornadoHandler):

    def get(self):
        if not firenado.conf.session['enabled']:
            self.write("Session is disabled as supposed to be.<br>")
            if self.session is None:
                self.write("Handler session is None.")
            else:
                self.write("Handler session isn't None. Something wrong "
                           "happened")
        else:
            self.write("Session is enabled. Something wrong happened.")
